@@ -330,7 +330,7 @@ FindAllMarkers <- function(
   object,
   genes.use = NULL,
   logfc.threshold = 0.25,
-  test.use = "bimod",
+  test.use = "wilcox",
   min.pct = 0.1,
   min.diff.pct = -Inf,
   print.bar = TRUE,
@@ -444,7 +444,7 @@ FindMarkersNode <- function(
   tree.use = NULL,
   genes.use = NULL,
   logfc.threshold = 0.25,
-  test.use = "bimod",
+  test.use = "wilcox",
   assay.type = "RNA",
   ...
 ) {
@@ -518,7 +518,7 @@ FindAllMarkersNode <- function(
   node = NULL,
   genes.use = NULL,
   logfc.threshold = 0.25,
-  test.use = "bimod",
+  test.use = "wilcox",
   min.pct = 0.1,
   min.diff.pct = 0.05,
   print.bar = TRUE,
@@ -609,19 +609,23 @@ FindAllMarkersNode <- function(
 #'
 #' @param object Seurat object
 #' @param ident.1 Identity class to define markers for
-#' @param ident.2 A second identity class for comparison. If NULL (default) - use all other cells
-#' for comparison.
+#' @param ident.2 A second identity class for comparison. If NULL (default) -
+#' use all other cells for comparison.
 #' @param grouping.var grouping variable
 #' @param assay.type Type of assay to fetch data for (default is RNA)
+#' @param meta.method method for combining p-values. See MetaDE::MetaDE.pvalue
+#' for details.
 #' @param \dots parameters to pass to FindMarkers
 #'
-#' @return Matrix containing a ranked list of putative conserved markers, and associated statistics
-#' (p-values within each group and a combined p-value (fisher_pval), percentage of cells expressing
-#' the marker, average differences)
+#' @return Matrix containing a ranked list of putative conserved markers, and
+#' associated statistics (p-values within each group and a combined p-value
+#' (such as Fishers combined p-value or others from the MetaDE package),
+#' percentage of cells expressing the marker, average differences)
 #'
 #' @export
 #'
 #' @examples
+#' \dontrun{
 #' pbmc_small
 #' # Create a simulated grouping variable
 #' pbmc_small@meta.data$groups <- sample(
@@ -630,6 +634,7 @@ FindAllMarkersNode <- function(
 #'   replace = TRUE
 #' )
 #' FindConservedMarkers(pbmc_small, ident.1 = 1, ident.2 = 2, grouping.var = "groups")
+#' }
 #'
 FindConservedMarkers <- function(
   object,
@@ -637,8 +642,12 @@ FindConservedMarkers <- function(
   ident.2 = NULL,
   grouping.var,
   assay.type = "RNA",
+  meta.method = "minP",
   ...
 ) {
+  if (!'MetaDE' %in% rownames(x = installed.packages())) {
+    stop("Please install MetaDE (note Bioconductor dependencies) - learn more at https://cran.r-project.org/web/packages/MetaDE/index.html")
+  }
   object.var <- FetchData(object = object, vars.all = grouping.var)
   object <- SetIdent(
     object = object,
@@ -646,27 +655,21 @@ FindConservedMarkers <- function(
     ident.use = paste(object@ident, object.var[, 1], sep = "_")
   )
   levels.split <- names(x = sort(x = table(object.var[, 1])))
-  if (length(x = levels.split) != 2) {
-    stop(
-      paste0(
-        "There are not two options for ",
-        grouping.var,
-        ". \n Current groups include: ",
-        paste(levels.split, collapse = ", ")
-      )
-    )
-  }
+  num.groups <- length(levels.split)
   cells <- list()
-  for (i in 1:2) {
+  for (i in 1:num.groups) {
     cells[[i]] <- rownames(
       x = object.var[object.var[, 1] == levels.split[i], , drop = FALSE]
     )
   }
   marker.test <- list()
   # do marker tests
-  for (i in 1:2) {
+  for (i in 1:num.groups) {
     level.use <- levels.split[i]
     ident.use.1 <- paste(ident.1, level.use, sep = "_")
+    if(!ident.use.1 %in% object@ident) {
+      stop(paste0("Identity: ", ident.1, " not present in group ", level.use))
+    }
     cells.1 <- WhichCells(object = object, ident = ident.use.1)
     if (is.null(x = ident.2)) {
       cells.2 <- setdiff(x = cells[[i]], y = cells.1)
@@ -687,6 +690,9 @@ FindConservedMarkers <- function(
       ),
       file = stderr()
     )
+    if(!ident.use.2 %in% object@ident) {
+      stop(paste0("Identity: ", ident.2, " not present in group ", level.use))
+    }
     marker.test[[i]] <- FindMarkers(
       object = object,
       assay.type = assay.type,
@@ -695,12 +701,9 @@ FindConservedMarkers <- function(
       ...
     )
   }
-  genes.conserved <- intersect(
-    x = rownames(x = marker.test[[1]]),
-    y = rownames(x = marker.test[[2]])
-  )
+  genes.conserved <- Reduce(intersect, lapply(marker.test, FUN = function(x) rownames(x)))
   markers.conserved <- list()
-  for (i in 1:2) {
+  for (i in 1:num.groups) {
     markers.conserved[[i]] <- marker.test[[i]][genes.conserved, ]
     colnames(x = markers.conserved[[i]]) <- paste(
       levels.split[i],
@@ -708,19 +711,19 @@ FindConservedMarkers <- function(
       sep="_"
     )
   }
-  markers.combined <- cbind(markers.conserved[[1]], markers.conserved[[2]])
+  markers.combined <- Reduce(cbind, markers.conserved)
   pval.codes <- paste(levels.split, "p_val", sep = "_")
   markers.combined$max_pval <- apply(
     X = markers.combined[, pval.codes],
     MARGIN = 1,
     FUN = max
   )
-  markers.combined$fisher_pval <- apply(
-    X = markers.combined[, pval.codes],
-    MARGIN = 1,
-    FUN = FisherIntegrate
-  )
-  markers.combined <- markers.combined[order(markers.combined$fisher_pval), ]
+  combined.pval <- MetaDE::MetaDE.pvalue(x = list(p = markers.combined[, pval.codes]),
+                                 meta.method = meta.method)$meta.analysis
+  combined.pval <- cbind(combined.pval$pval, combined.pval$FDR)
+  colnames(combined.pval) <- paste0(colnames(combined.pval), c(rep("_p_val", length(meta.method)), rep("_FDR", length(meta.method))))
+  markers.combined$combined_pval <- combined.pval
+  markers.combined <- markers.combined[order(markers.combined$combined_pval[,1]), ]
   return(markers.combined)
 }
 
