@@ -1,6 +1,3 @@
-
-
-
 #' Run Principal Component Analysis on gene expression using IRLBA
 #'
 #' Run a PCA dimensionality reduction. For details about stored PCA calculation
@@ -21,6 +18,8 @@
 #' @param reduction.name dimensional reduction name, specifies the position in the object$dr list. pca by default
 #' @param reduction.key dimensional reduction key, specifies the string before the number for the dimension names. PC by default
 #' @param assay.type Data type, RNA by default. Can be changed for multimodal
+#' @param seed.use Set a random seed. By default, sets the seed to 42. Setting
+#' NULL will not set a seed.
 #' @param \dots Additional arguments to be passed to IRLBA
 #'
 #'@importFrom irlba irlba
@@ -56,8 +55,12 @@ RunPCA <- function(
   reduction.name = "pca",
   reduction.key = "PC",
   assay.type="RNA",
+  seed.use = 42,
   ...
 ) {
+  if (!is.null(seed.use)) {
+    set.seed(seed = seed.use)
+  }
   data.use <- PrepDR(
     object = object,
     genes.use = pc.genes,
@@ -275,9 +278,9 @@ RunTSNE <- function(
     )[, dims.use]
   }
   if (! is.null(x = genes.use)) {
-      data.use <- t(PrepDR(
-        object = object,
-        genes.use = genes.use))
+    data.use <- t(PrepDR(
+      object = object,
+      genes.use = genes.use))
   }
   set.seed(seed = seed.use)
   if (do.fast) {
@@ -481,7 +484,7 @@ ProjectPCA <- function(
 #' that are also present in both objects.
 #' @param scale.data Use the scaled data from the object
 #' @param rescale.groups Rescale each set of cells independently
-#' @param ... Extra parameters (passed onto MergeSeurat in case with two objects 
+#' @param ... Extra parameters (passed onto MergeSeurat in case with two objects
 #' passed, passed onto ScaleData in case with single object and rescale.groups
 #' set to TRUE)
 #'
@@ -609,14 +612,20 @@ RunCCA <- function(
   cat("Running CCA\n", file = stderr())
 
   cca.results <- CanonCor(
-      mat1 = data.use1,
-      mat2 = data.use2,
-      standardize = TRUE,
-      k = num.cc
+    mat1 = data.use1,
+    mat2 = data.use2,
+    standardize = TRUE,
+    k = num.cc
   )
   cca.data <- rbind(cca.results$u, cca.results$v)
   colnames(x = cca.data) <- paste0("CC", 1:num.cc)
   rownames(cca.data) <- c(colnames(data.use1), colnames(data.use2))
+  cca.data <- apply(cca.data, MARGIN = 2, function(x){
+    if(sign(x[1]) == -1) {
+      x <- x * -1
+    }
+    return(x)
+  })
   # wipe old CCA slot
   object@dr$cca <- NULL
   if (! missing(x = object2)) {
@@ -632,6 +641,14 @@ RunCCA <- function(
     combined.object <- ScaleData(object = combined.object)
     combined.object@scale.data[is.na(x = combined.object@scale.data)] <- 0
     combined.object@var.genes <- genes.use
+    if("add.cell.id1" %in% names(list(...)) && "add.cell.id2" %in% names(list(...))) {
+      o1.idx <- 1:length(object@cell.names)
+      o2.idx <- (length(object@cell.names) + 1):(length(object@cell.names) + length(object2@cell.names))
+      rownames(cca.data)[o1.idx] <-
+        paste0(list(...)$add.cell.id1, "_", rownames(cca.data)[o1.idx])
+      rownames(cca.data)[o2.idx] <-
+        paste0(list(...)$add.cell.id2, "_", rownames(cca.data)[o2.idx])
+    }
     combined.object <- SetDimReduction(
       object = combined.object,
       reduction.type = "cca",
@@ -720,363 +737,141 @@ RunCCA <- function(
   }
 }
 
-#' Calculate the ratio of variance explained by ICA or PCA to CCA
+#' Perform Canonical Correlation Analysis with more than two groups
 #'
-#' @param object Seurat object
-#' @param reduction.type type of dimensional reduction to compare to CCA (pca,
-#' pcafast, ica)
-#' @param grouping.var variable to group by
-#' @param dims.use Vector of dimensions to project onto (default is the 1:number
-#'  stored for cca)
+#' Runs a canonical correlation analysis
 #'
-#' @return Returns Seurat object with ratio of variance explained stored in
-#' object@@meta.data$var.ratio
+#' @param object.list List of Seurat objects
+#' @param genes.use Genes to use in mCCA.
+#' @param niter Number of iterations to perform. Set by default to 25.
+#' @param num.ccs Number of canonical vectors to calculate
+#' @param standardize standardize scale.data matrices to be centered (mean zero)
+#' and scaled to have a standard deviation of 1.
+#' @return Returns a combined Seurat object with the CCA stored in the @@dr$cca slot.
 #' @export
-#'
-#' @examples
-#' pbmc_small
-#' # Requires CCA to have previously been run
-#' # As CCA requires two datasets, we will split our test object into two just for this example
-#' pbmc1 <- SubsetData(pbmc_small,cells.use = pbmc_small@cell.names[1:40])
-#' pbmc2 <- SubsetData(pbmc_small,cells.use = pbmc_small@cell.names[41:80])
-#' pbmc1@meta.data$group <- "group1"
-#' pbmc2@meta.data$group <- "group2"
-#' pbmc_cca <- RunCCA(pbmc1,pbmc2)
-#' pbmc_cca <- CalcVarExpRatio(pbmc_cca,reduction.type = "pca", grouping.var = "group", dims.use = 1:5)
-#'
-CalcVarExpRatio <- function(
-  object,
-  reduction.type = "pca",
-  grouping.var,
-  dims.use
-) {
-  if (missing(x = grouping.var)) {
-    stop("Need to provide grouping variable")
+RunMultiCCA <- function(object.list, genes.use, niter = 25, num.ccs = 1, standardize = TRUE){
+  set.seed(42)
+  if(length(object.list) < 3){
+    stop("Must give at least 3 objects/matrices for MultiCCA")
   }
-  if (missing(x = dims.use)) {
-    dims.use <- 1:ncol(x = GetCellEmbeddings(object = object, reduction.type = "cca"))
-  }
-  parameters.to.store <- as.list(environment(), all = TRUE)[names(formals("CalcVarExpRatio"))]
-  object <- SetCalcParams(object = object,
-                          calculation = "CalcVarExpRatio",
-                          ... = parameters.to.store)
-  groups <- as.vector(x = unique(x = FetchData(
-    object = object,
-    vars.all = grouping.var
-  )[, 1]))
-  genes.use <- rownames(x = GetGeneLoadings(object = object, reduction.type = "cca"))
-  var.ratio <- data.frame()
-  for (group in groups) {
-    cat(paste("Calculating for", group, "\n"), file = stderr())
-    group.cells <- WhichCells(
-      object = object,
-      subset.name = grouping.var,
-      accept.value = group
-    )
-    cat(paste("\t Separating", group, "cells\n"), file = stderr())
-    group.object <- SubsetData(object = object, cells.use = group.cells)
-    cat("\t Running Dimensional Reduction \n", file = stderr())
-    ldp.cca <- CalcLDProj(
-      object = group.object,
-      reduction.type = "cca",
-      dims.use = dims.use,
-      genes.use = genes.use
-    )
-    group.object <- CalcProjectedVar(
-      object = group.object,
-      low.dim.data = ldp.cca,
-      reduction.type = "cca",
-      dims.use = dims.use,
-      genes.use = genes.use
-    )
-    if (reduction.type == "pca") {
-      temp.matrix=PrepDR(group.object,genes.use = genes.use)
-      group.object <- RunPCA(
-        object = group.object,
-        pc.genes = genes.use,
-        do.print = FALSE,
-        center=rowMeans(temp.matrix)
-      )
-      ldp.pca <- CalcLDProj(
-        object = group.object,
-        reduction.type = "pca",
-        dims.use = dims.use,
-        genes.use = genes.use
-      )
-      group.object <- CalcProjectedVar(
-        object = group.object,
-        low.dim.data = ldp.pca,
-        reduction.type = "pca",
-        dims.use = dims.use,
-        genes.use = genes.use
-      )
-      group.var.ratio <- group.object@meta.data[, "cca.var", drop = FALSE] /
-        group.object@meta.data[, "pca.var", drop = FALSE]
-    } else if (reduction.type == "ica") {
-      group.object <- RunICA(
-        object = group.object,
-        ic.genes = genes.use,
-        print.results = FALSE
-      )
-      ldp.ica <- CalcLDProj(
-        object = group.object,
-        reduction.type = "ica",
-        dims.use = dims.use,
-        genes.use = genes.use
-      )
-      group.object <- CalcProjectedVar(
-        object = group.object,
-        low.dim.data = ldp.ica,
-        reduction.type = "ica",
-        dims.use = dims.use,
-        genes.use = genes.use
-      )
-      group.var.ratio <- group.object@meta.data[, "cca.var", drop = FALSE] /
-        group.object@meta.data[, "ica.var", drop = FALSE]
-    } else {
-        stop(paste("reduction.type", reduction.type, "not supported"))
-    }
-    var.ratio <- rbind(var.ratio, group.var.ratio)
-  }
-  var.ratio$cell.name <- rownames(x = var.ratio)
-  eval(expr = parse(text = paste0(
-    "object@meta.data$var.ratio.",
-    reduction.type,
-    "<- NULL"
-  )))
-  colnames(x = var.ratio) <- c(
-    paste0("var.ratio.", reduction.type),
-    "cell.name"
-  )
-  object@meta.data$cell.name <- rownames(x = object@meta.data)
-  object@meta.data <- merge(x = object@meta.data, y = var.ratio, by = "cell.name")
-  rownames(x = object@meta.data) <- object@meta.data$cell.name
-  object@meta.data$cell.name <- NULL
-  return(object)
-}
-
-#' Align subspaces using dynamic time warping (DTW)
-#'
-#' Aligns subspaces so that they line up across grouping variable (only
-#' implemented for case with 2 categories in grouping.var)
-#'
-#'
-#' @param object Seurat object
-#' @param reduction.type reduction to align scores for
-#' @param grouping.var Name of the grouping variable for which to align the scores
-#' @param dims.align Dims to align, default is all
-#' @param num.genes Number of genes to use in construction of "metagene"
-#' @param show.plots show debugging plots
-#'
-#' @return Returns Seurat object with the dims aligned, stored in
-#'  object@@dr$reduction.type.aligned
-#'
-#' @importFrom dtw dtw
-#' @importFrom graphics points
-#' @importFrom stats quantile density
-#' @importFrom pbapply pbapply
-#' @importFrom graphics par plot lines
-#'
-#' @export
-#'
-#' @examples
-#' \dontrun{
-#' pbmc_small
-#' # Requires CCA to have previously been run
-#' # As CCA requires two datasets, we will split our test object into two just for this example
-#' pbmc1 <- SubsetData(pbmc_small,cells.use = pbmc_small@cell.names[1:40])
-#' pbmc2 <- SubsetData(pbmc_small,cells.use = pbmc_small@cell.names[41:80])
-#' pbmc1@meta.data$group <- "group1"
-#' pbmc2@meta.data$group <- "group2"
-#' pbmc_cca <- RunCCA(pbmc1,pbmc2)
-#' pbmc_cca <- AlignSubspace(pbmc_cca,reduction.type = "cca", grouping.var = "group", dims.align = 1:2)
-#' }
-#'
-AlignSubspace <- function(
-  object,
-  reduction.type,
-  grouping.var,
-  dims.align,
-  num.genes = 30,
-  show.plots = FALSE
-) {
-  parameters.to.store <- as.list(environment(), all = TRUE)[names(formals("AlignSubspace"))]
-  object <- SetCalcParams(
-    object = object,
-    calculation = paste0("AlignSubspace.", reduction.type),
-    ... = parameters.to.store
-  )
-  ident.orig <- object@ident
-  object <- SetAllIdent(object = object, id = grouping.var)
-  levels.split <- names(x = sort(x = table(object@ident)))
-  if (length(x = levels.split) != 2) {
-    stop(paste0(
-      "There are not two options for ",
-      grouping.var,
-      ". \n Current groups include: ",
-      paste(levels.split, collapse = ", ")
-    ))
-  }
-  objects <- list(
-    SubsetData(object = object, ident.use = levels.split[1]),
-    SubsetData(object = object, ident.use = levels.split[2])
-  )
-  object@ident <- ident.orig
-  cc.loadings <- list()
-  scaled.data <- list()
-  cc.embeds <- list()
-  for (i in 1:2) {
-    cat(paste0("Rescaling group ", i, "\n"), file = stderr())
-    objects[[i]] <- ScaleData(object = objects[[i]])
-    objects[[i]]@scale.data[is.na(x = objects[[i]]@scale.data)] <- 0
-    objects[[i]] <- ProjectDim(
-      object = objects[[i]],
-      reduction.type = reduction.type,
-      do.print = FALSE
-    )
-    cc.loadings[[i]] <- GetGeneLoadings(
-      object = objects[[i]],
-      reduction.type = reduction.type,
-      use.full = TRUE
-    )
-    cc.embeds[[i]] <- GetCellEmbeddings(
-      object = objects[[i]],
-      reduction.type = reduction.type
-    )
-    scaled.data[[i]] <- objects[[i]]@scale.data
-  }
-  cc.embeds.both <- GetCellEmbeddings(object = object, reduction.type = reduction.type)
-  colnames(cc.embeds.both) <- paste0("A", colnames(x = cc.embeds.both))
-  cc.embeds.orig <- cc.embeds.both
-  for (cc.use in dims.align) {
-    cat(paste0("Aligning dimension ", cc.use, "\n"), file = stderr())
-    genes.rank <- data.frame(
-      rank(x = abs(x = cc.loadings[[1]][, cc.use])),
-      rank(x = abs(x = cc.loadings[[2]][, cc.use])),
-      cc.loadings[[1]][, cc.use],
-      cc.loadings[[2]][, cc.use]
-    )
-    genes.rank$min <- apply(X = genes.rank[,1:2], MARGIN = 1, FUN = min)
-    genes.rank <- genes.rank[order(genes.rank$min, decreasing = TRUE), ]
-    genes.top <- rownames(x = genes.rank)[1:200]
-    bicors <- list()
-    for (i in 1:2) {
-      cc.vals <- cc.embeds[[i]][, cc.use]
-      bicors[[i]] <- pbsapply(
-        X = genes.top,
-        FUN = function(x) {
-          return(BiweightMidcor(x = cc.vals, y = scaled.data[[i]][x, ]))
-        }
-      )
-    }
-    genes.rank <- data.frame(
-      rank(x = abs(x = bicors[[1]])),
-      rank(x = abs(x = bicors[[2]])),
-      bicors[[1]],
-      bicors[[2]]
-    )
-    genes.rank$min <- apply(X = abs(x = genes.rank[, 1:2]), MARGIN = 1, FUN = min)
-    genes.rank <- genes.rank[order(genes.rank$min, decreasing = TRUE), ]
-    genes.use <- rownames(x = genes.rank)[1:num.genes]
-    metagenes <- list()
-    multvar.data <- list()
-    for (i in 1:2) {
-      scaled.use <- sweep(
-        x = scaled.data[[i]][genes.use, ],
-        MARGIN = 1,
-        STATS = sign(x = genes.rank[genes.use, i + 2]),
-        FUN = "*"
-      )
-      scaled.use <- scaled.use[, names(x = sort(x = cc.embeds[[i]][, cc.use]))]
-      metagenes[[i]] <- apply(
-        X = scaled.use[genes.use, ],
-        MARGIN = 2,
-        FUN = mean,
-        remove.na = TRUE
-      )
-      metagenes[[i]] <- (
-        cc.loadings[[i]][genes.use, cc.use] %*% scaled.data[[i]][genes.use, ]
-      )[1, colnames(x = scaled.use)]
-    }
-
-    mean.difference <- mean(x = ReferenceRange(x = metagenes[[1]])) -
-      mean(x = ReferenceRange(x = metagenes[[2]]))
-    metric.use <- "Euclidean"
-    align.1 <- ReferenceRange(x = metagenes[[1]])
-    align.2 <- ReferenceRange(x = metagenes[[2]])
-    a1q <- sapply(
-      X = seq(from = 0, to = 1, by = 0.001),
-      FUN = function(x) {
-        return(quantile(x = align.1, probs = x))
+  mat.list <- list()
+  if(class(object.list[[1]]) == "seurat"){
+    if (missing(x = genes.use)) {
+      genes.use <- c()
+      for(obj in object.list){
+        genes.use <- c(genes.use, obj@var.genes)
       }
-    )
-    a2q <- sapply(
-      X = seq(from = 0, to = 1, by = 0.001),
-      FUN = function(x) {
-        quantile(x = align.2, probs = x)
+      genes.use <- unique(genes.use)
+      if (length(x = genes.use) == 0) {
+        stop("No variable genes present. Run MeanVarPlot and retry")
       }
-    )
-    iqr <- (a1q - a2q)[100:900]
-    iqr.x <- which.min(x = abs(x = iqr))
-    iqrmin <- iqr[iqr.x]
-    if (show.plots) {
-      print(iqrmin)
     }
-    align.2 <- align.2 + iqrmin
-    alignment <- dtw(
-      x = align.1,
-      y = align.2,
-      keep.internals = TRUE,
-      dist.method = metric.use
-    )
-    alignment.map <- data.frame(alignment$index1, alignment$index2)
-    alignment.map$cc_data1 <- sort(cc.embeds[[1]][, cc.use])[alignment$index1]
-    alignment.map$cc_data2 <- sort(cc.embeds[[2]][, cc.use])[alignment$index2]
-    alignment.map.orig <- alignment.map
-    alignment.map <- alignment.map[! duplicated(x = alignment.map$alignment.index1), ]
-    cc.embeds.both[names(x = sort(x = cc.embeds[[1]][, cc.use])), cc.use] <- alignment.map$cc_data2
-    if (show.plots) {
-      par(mfrow = c(3, 2))
-      plot(x = ReferenceRange(x = metagenes[[1]]), main = cc.use)
-      plot(x = ReferenceRange(x = metagenes[[2]]))
-      plot(
-        x = ReferenceRange(x = metagenes[[1]])[(alignment.map.orig$alignment.index1)],
-        pch = 16
-      )
-      points(
-        x = ReferenceRange(metagenes[[2]])[(alignment.map.orig$alignment.index2)],
-        col = "red",
-        pch = 16,
-        cex = 0.4
-      )
-      plot(x = density(x = alignment.map$cc_data2))
-      lines(x = density(x = sort(x = cc.embeds[[2]][, cc.use])), col = "red")
-      plot(x = alignment.map.orig$cc_data1)
-      points(x = alignment.map.orig$cc_data2, col = "red")
+    for(obj in object.list) {
+      genes.use <- CheckGenes(data.use = obj@scale.data, genes.use = genes.use)
+    }
+    for(i in 1:length(object.list)){
+      mat.list[[i]] <- object.list[[i]]@scale.data[genes.use, ]
     }
   }
-  new.type <- paste0(reduction.type, ".aligned")
-  new.key <- paste0(
-    "A",
-    GetDimReduction(
-      object = object,
-      reduction.type = reduction.type,
-      slot = "key"
-    )
-  )
-  object <- SetDimReduction(
-    object = object,
-    reduction.type = new.type,
+  else{
+    stop("input data not Seurat objects")
+  }
+  num.sets <- length(mat.list)
+  if(standardize){
+    for (i in 1:num.sets){
+      mat.list[[i]] <- Standardize(mat.list[[i]], display_progress = F)
+    }
+  }
+  ws <- list()
+  for (i in 1:num.sets){
+    ws[[i]] <- irlba(mat.list[[i]], nv = num.ccs)$v[, 1:num.ccs, drop = F]
+  }
+  ws.init <- ws
+  ws.final <- list()
+  cors <- NULL
+  for(i in 1:length(ws)){
+    ws.final[[i]] <- matrix(0, nrow=ncol(mat.list[[i]]), ncol=num.ccs)
+  }
+  for (cc in 1:num.ccs){
+    print(paste0("Computing CC ", cc))
+    ws <- list()
+    for (i in 1:length(ws.init)){
+      ws[[i]] <- ws.init[[i]][, cc]
+    }
+    cur.iter <- 1
+    crit.old <- -10
+    crit <- -20
+    storecrits <- NULL
+    while(cur.iter <= niter && abs(crit.old - crit)/abs(crit.old) > 0.001 && crit.old !=0){
+      crit.old <- crit
+      crit <- GetCrit(mat.list, ws, num.sets)
+      storecrits <- c(storecrits, crit)
+      cur.iter <- cur.iter + 1
+      for(i in 1:num.sets){
+        ws[[i]] <- UpdateW(mat.list, i, num.sets, ws, ws.final)
+      }
+    }
+    for(i in 1:length(ws)){
+      ws.final[[i]][, cc] <- ws[[i]]
+    }
+    cors <- c(cors, GetCors(mat.list, ws, num.sets))
+  }
+  results <- list(ws=ws.final, ws.init=ws.init, num.sets = num.sets, cors=cors)
+  combined.object <- object.list[[1]]
+  for(i in 2:length(object.list)){
+    combined.object <- MergeSeurat(object1 = combined.object, object2 = object.list[[i]], do.scale = F, do.center = F, do.normalize = F)
+  }
+  combined.object <- NormalizeData(combined.object)
+  combined.object@meta.data$orig.ident <- sapply(combined.object@cell.names, ExtractField, 1)
+  combined.object <- ScaleData(object = combined.object)
+  combined.object@scale.data[is.na(x = combined.object@scale.data)] <- 0
+  combined.object@var.genes <- genes.use
+  cca.data <- results$ws[[1]]
+  for(i in 2:length(object.list)){
+    cca.data <- rbind(cca.data, results$ws[[i]])
+  }
+  rownames(cca.data) <- colnames(combined.object@data)
+  cca.data <- apply(cca.data, MARGIN = 2, function(x){
+    if(sign(x[1]) == -1) {
+      x <- x * -1
+    }
+    return(x)
+  })
+  combined.object <- SetDimReduction(
+    object = combined.object,
+    reduction.type = "cca",
     slot = "cell.embeddings",
-    new.data = scale(x = cc.embeds.both)
+    new.data = cca.data
   )
-  object <- SetDimReduction(
-    object = object,
-    reduction.type = new.type,
+  combined.object <- SetDimReduction(
+    object = combined.object,
+    reduction.type = "cca",
     slot = "key",
-    new.data = new.key
+    new.data = "CC"
   )
-  return(object)
+  combined.object <- ProjectDim(
+    object = combined.object,
+    reduction.type = "cca",
+    do.print = FALSE
+  )
+  combined.object <- SetDimReduction(
+    object = combined.object,
+    reduction.type = "cca",
+    slot = "gene.loadings",
+    new.data = GetGeneLoadings(
+      object = combined.object,
+      reduction.type = "cca",
+      use.full = TRUE,
+      genes.use = genes.use
+    )
+  )
+  parameters.to.store <- as.list(environment(), all = TRUE)[names(formals("RunMultiCCA"))]
+  parameters.to.store$object.list <- NULL
+  combined.object <- SetCalcParams(object = combined.object,
+                                   calculation = "RunMultiCCA",
+                                   ... = parameters.to.store
+  )
+  return(combined.object)
 }
 
 #' Run diffusion map
@@ -1099,7 +894,7 @@ AlignSubspace <- function(
 #' @return Returns a Seurat object with a diffusion map
 #'
 #' @import diffusionMap
-#' @importFrom stats dist
+#' @importFrom stats dist quantile
 #'
 #' @export
 #'
